/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.2 Straßensubsegmentanalyse
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.subsegana.subsegana;

import java.util.Collection;

import com.bitctrl.Constants;

import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.iav.subsegana.subsegana.events.MesswertDispatcher;
import de.bsvrz.sys.funclib.bitctrl.daf.BetriebsmeldungDaten;
import de.bsvrz.sys.funclib.bitctrl.daf.LogTools;
import de.bsvrz.sys.funclib.bitctrl.modell.DatensatzUpdateEvent;
import de.bsvrz.sys.funclib.bitctrl.modell.DatensatzUpdateListener;
import de.bsvrz.sys.funclib.bitctrl.modell.Datum;
import de.bsvrz.sys.funclib.bitctrl.modell.MesswertDatum;
import de.bsvrz.sys.funclib.bitctrl.modell.ObjektFactory;
import de.bsvrz.sys.funclib.bitctrl.modell.OnlineDatensatz;
import de.bsvrz.sys.funclib.bitctrl.modell.SystemObjekt;
import de.bsvrz.sys.funclib.bitctrl.modell.umfelddaten.objekte.UfdsHelligkeit;
import de.bsvrz.sys.funclib.bitctrl.modell.umfelddaten.objekte.UfdsNiederschlagsIntensitaet;
import de.bsvrz.sys.funclib.bitctrl.modell.umfelddaten.objekte.UfdsSichtWeite;
import de.bsvrz.sys.funclib.bitctrl.modell.umfelddaten.objekte.UfdsWindGeschwindigkeitMittelWert;
import de.bsvrz.sys.funclib.bitctrl.modell.umfelddaten.objekte.UfdsWindRichtung;
import de.bsvrz.sys.funclib.bitctrl.modell.umfelddaten.objekte.UmfeldDatenMessStelle;
import de.bsvrz.sys.funclib.bitctrl.modell.umfelddaten.onlinedaten.OdUfdmsGlaette;
import de.bsvrz.sys.funclib.bitctrl.modell.umfelddaten.onlinedaten.OdUfdsHelligkeit;
import de.bsvrz.sys.funclib.bitctrl.modell.umfelddaten.onlinedaten.OdUfdsNiederschlagsIntensitaet;
import de.bsvrz.sys.funclib.bitctrl.modell.umfelddaten.onlinedaten.OdUfdsSichtWeite;
import de.bsvrz.sys.funclib.bitctrl.modell.umfelddaten.onlinedaten.OdUfdsWindGeschwindigkeitMittelWert;
import de.bsvrz.sys.funclib.bitctrl.modell.umfelddaten.onlinedaten.OdUfdsWindRichtung;
import de.bsvrz.sys.funclib.bitctrl.modell.verkehr.objekte.MessQuerschnittAllgemein;
import de.bsvrz.sys.funclib.bitctrl.modell.verkehr.onlinedaten.OdVerkehrsDatenKurzZeitMq;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Empfängt die benötigten Analysewerte. Meldet sich für die Analysewerte der
 * Systemobjekte als Empfänger an und benachrichtigt Fuzzyfizierun über neue
 * Messwerte.
 *
 * @author BitCtrl Systems GmbH, Falko Schumann
 */
public class MesswertEmpfaenger extends MesswertDispatcher
implements DatensatzUpdateListener {

	/** Logger. */
	private final Debug log = Debug.getLogger();

	/** Wird am Ende des Konstruktors auf {@code true} gesetzt. */
	private boolean bereit = false;

	/**
	 * Meldet sich für die Analysedaten der Systemobjekte als Empfänger an.
	 *
	 * @param objekte
	 *            eine Menge von Systemobjekten.
	 */
	MesswertEmpfaenger(final Collection<SystemObjekt> objekte) {
		int anzahl = 0;

		LogTools.log(log, SubSegAnaMsg.InfoStarteMesswertEmpfaenger);

		// Auf Messwerte anmelden
		for (final SystemObjekt so : objekte) {
			if (so instanceof MessQuerschnittAllgemein) {
				anzahl += anmeldenEmpfaenger(so,
						OdVerkehrsDatenKurzZeitMq.class,
						OdVerkehrsDatenKurzZeitMq.Aspekte.Analyse.getAspekt());
			} else if (so instanceof UfdsHelligkeit) {
				anzahl += anmeldenEmpfaenger(so, OdUfdsHelligkeit.class,
						OdUfdsHelligkeit.Aspekte.MessWertErsetzung.getAspekt());
			} else if (so instanceof UfdsNiederschlagsIntensitaet) {
				anzahl += anmeldenEmpfaenger(so,
						OdUfdsNiederschlagsIntensitaet.class,
						OdUfdsNiederschlagsIntensitaet.Aspekte.MessWertErsetzung
								.getAspekt());
			} else if (so instanceof UfdsWindGeschwindigkeitMittelWert) {
				anzahl += anmeldenEmpfaenger(so,
						OdUfdsWindGeschwindigkeitMittelWert.class,
						OdUfdsWindGeschwindigkeitMittelWert.Aspekte.MessWertErsetzung
								.getAspekt());
			} else if (so instanceof UfdsSichtWeite) {
				anzahl += anmeldenEmpfaenger(so, OdUfdsSichtWeite.class,
						OdUfdsSichtWeite.Aspekte.MessWertErsetzung.getAspekt());
			} else if (so instanceof UfdsWindRichtung) {
				anzahl += anmeldenEmpfaenger(so, OdUfdsWindRichtung.class,
						OdUfdsWindRichtung.Aspekte.MessWertErsetzung
								.getAspekt());
			} else if (so instanceof UmfeldDatenMessStelle) {
				anzahl += anmeldenEmpfaenger(so, OdUfdmsGlaette.class,
						OdUfdmsGlaette.Aspekte.Prognose.getAspekt());
			} else {
				LogTools.log(log,
						new BetriebsmeldungDaten(so.getSystemObject()),
						SubSegAnaMsg.FinerIgnoriereUnbekanntesSystemObject, so);
			}
		}

		ObjektFactory.getInstanz().getVerbindung()
				.sleep(3 * Constants.MILLIS_PER_SECOND);
		synchronized (this) {
			bereit = true;
		}
		LogTools.log(log, SubSegAnaMsg.ConfigMesswertEmpfaengerBereit, anzahl);
	}

	@Override
	public void datensatzAktualisiert(final DatensatzUpdateEvent event) {
		synchronized (this) {
			// Daten werden erst verarbeitet, wenn Konstruktor fertig ist.
			if (!bereit) {
				return;
			}
		}

		OnlineDatensatz<? extends MesswertDatum> od;

		/*
		 * Auch eine Prüfung mit instanceof kann die Warnung nicht beheben, weil
		 * die man nur gegen OnlineDatensatz<?> und nicht gegen
		 * OnlineDatensatz<? extends MesswertDatum> prüfen kann.
		 *
		 * Der Cast ist aber immer erfolgreich, solange die Listener-Anmeldungen
		 * nur auf entsprechend passende Daten gemacht werden. Dies kann im
		 * Konstruktor per Code Review gerprüft werden, dass für alle
		 * Anmeldungen die Methode anmeldenEmpfaenger() verwendet wird, wo der
		 * Generic OnlineDatensatz<? extends MesswertDatum> sichergestellt wird.
		 */
		od = (OnlineDatensatz<? extends MesswertDatum>) event.getDatensatz();
		final SystemObjekt so = event.getObjekt();
		final Datum datum = event.getDatum();
		LogTools.log(log, new BetriebsmeldungDaten(so.getSystemObject()),
				SubSegAnaMsg.FineEmpfangeMesswert, so, datum);
		fireNeuerMesswert(od.getObjekt(), od, (MesswertDatum) datum);
	}

	/**
	 * Meldet sich als Empf&auml;ger eines Datensatzes eines Systemobjekts an.
	 *
	 * @param so
	 *            ein Systemobjekt.
	 * @param datensatz
	 *            der zu empfangene Datensatz.
	 * @param asp
	 *            der Aspekt unter dem Daten empfangen werden sollen.
	 * @return 1 beim erfolgreichen Anmelden, sonst 0.
	 */
	private int anmeldenEmpfaenger(final SystemObjekt so,
			final Class<? extends OnlineDatensatz<? extends MesswertDatum>> datensatz,
			final Aspect asp) {
		OnlineDatensatz<?> od;

		od = so.getOnlineDatensatz(datensatz);
		LogTools.log(log, new BetriebsmeldungDaten(so.getSystemObject()),
				SubSegAnaMsg.FinerAnmeldenEmpfaengerMesswert, so,
				od.getAttributGruppe(), asp);
		od.addUpdateListener(asp, this);

		return 1;
	}

}
