/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.2 Straßensubsegmentanalyse
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.subsegana.subsegana;

import static org.junit.Assert.assertTrue;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;

import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.iav.fuzzylib.fuzzylib.FuzzyVariable;
import de.bsvrz.iav.fuzzylib.fuzzylib.modell.FuzzyWert;
import de.bsvrz.sys.funclib.bitctrl.modell.MesswertDatum;
import de.bsvrz.sys.funclib.bitctrl.modell.OnlineDatensatz;
import de.bsvrz.sys.funclib.bitctrl.modell.SystemObjekt;

/**
 * Dieser Testfall testet die Fuzzyfizierung eines eines Messwerts an einem
 * Testobjekt.
 *
 * @author BitCtrl Systems GmbH, Falko Schumann
 * @param <T>
 *            der Typ des Messwertdatums.
 */
@SuppressWarnings("nls")
public class TestfallFuzzyfizierung<T extends MesswertDatum>
		extends Testfall<T> {

	/**
	 * Initialisiert das Objekt.
	 *
	 * @param objekt
	 *            das Systemobjekt an dem der Test durchgeführt wird.
	 * @param name
	 *            der Name des Wertes der getestet wird.
	 * @param messwert
	 *            der Datensatz des fuzzyfizierende Messwerts.
	 * @param messwertAsp
	 *            der Aspekt der zum Senden der Messwerte verwendet werden soll.
	 * @param fuzzyWert
	 *            der Datensatz des Fuzzywerts.
	 * @throws SQLException
	 *             bei einem Datenbankfehler.
	 */
	public TestfallFuzzyfizierung(final SystemObjekt objekt, final String name,
			final Class<? extends OnlineDatensatz<T>> messwert,
					final Aspect messwertAsp,
					final Class<? extends OnlineDatensatz<FuzzyWert>> fuzzyWert)
					throws SQLException {
		super(objekt, name, messwert, messwertAsp, fuzzyWert);
	}

	@Override
	protected SortedMap<Long, FuzzyVariable> abfragenSoll()
			throws SQLException {
		final Statement stat;
		final ResultSet rs;
		final String sql;
		final SortedMap<Long, FuzzyVariable> daten;

		getLog().fine(getName() + ": Lese Sollwerte aus Datenbank ...");

		daten = new TreeMap<>();
		stat = getConnection().createStatement();
		sql = "SELECT * FROM fuzzy WHERE wertname LIKE '" + getName() + "'";
		rs = stat.executeQuery(sql);

		while (rs.next()) {
			final FuzzyVariable fv;
			final String sollText;
			final String[] sollwerte;
			final List<String> termnamen;
			final Map<String, Float> terme;
			final Timestamp zeitstempel;

			zeitstempel = rs.getTimestamp("zeitstempel");
			sollText = rs.getString("soll");
			sollwerte = sollText.split("\\|");
			termnamen = new ArrayList<>();
			terme = new HashMap<>();
			for (final String t : sollwerte) {
				String termname;

				termname = t.split(":")[0];
				termnamen.add(termname);
				terme.put(termname, Float.valueOf(t.split(":")[1]));
			}

			fv = new FuzzyVariable(getName(),
					termnamen.toArray(new String[termnamen.size()]));
			for (final String t : termnamen) {
				fv.setTerm(t, terme.get(t));
			}
			daten.put(zeitstempel.getTime(), fv);
		}

		assertTrue(daten.size() > 0);
		return daten;
	}

	@Override
	protected SortedMap<Long, T> abfragenTestdaten() throws SQLException {
		final Statement stat;
		final ResultSet rs;
		final String sql;
		final SortedMap<Long, T> daten;

		getLog().fine(getName() + ": Lese Testdaten aus Datenbank ...");

		daten = new TreeMap<>();
		stat = getConnection().createStatement();
		sql = "SELECT * FROM fuzzy WHERE wertname LIKE '" + getName() + "'";
		rs = stat.executeQuery(sql);

		while (rs.next()) {
			final T datum;
			final Timestamp zeitstempel;
			final Double wert;

			zeitstempel = rs.getTimestamp("zeitstempel");
			datum = getMesswert().erzeugeDatum();
			daten.put(zeitstempel.getTime(), datum);

			// Wert im Datensatz setzen, wenn ungleich null
			wert = rs.getDouble("messwert");
			if (!rs.wasNull()) {
				datum.setWert(getName(), wert);
			}
		}

		assertTrue(daten.size() > 0);
		return daten;
	}

}
