/*
 * Segment 5 Intelligente Analyseverfahren, SWE 5.2 Straßensubsegmentanalyse
 * Copyright (C) 2007 BitCtrl Systems GmbH
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.iav.subsegana.subsegana.events;

import javax.swing.event.EventListenerList;

import de.bsvrz.sys.funclib.bitctrl.modell.MesswertDatum;
import de.bsvrz.sys.funclib.bitctrl.modell.OnlineDatensatz;
import de.bsvrz.sys.funclib.bitctrl.modell.SystemObjekt;

/**
 * Abstrakte Klasse die Messwerte verteilt.
 *
 * @author BitCtrl Systems GmbH, Falko Schumann
 */
public abstract class MesswertDispatcher {

	/** Liste angemeldeter Listener. */
	private final EventListenerList listeners = new EventListenerList();

	/**
	 * Registriert einen Listener.
	 *
	 * @param listener
	 *            Der neue Listener
	 */
	public void addMesswertListener(final MesswertListener listener) {
		listeners.add(MesswertListener.class, listener);
	}

	/**
	 * Entfernt einen Listener wieder aus der Liste registrierter Listener.
	 *
	 * @param listener
	 *            Listener der abgemeldet werden soll
	 */
	public void removeMesswertListener(final MesswertListener listener) {
		listeners.remove(MesswertListener.class, listener);
	}

	/**
	 * Informiert die angemeldeten Listener &uuml;ber die neuen Analysedaten.
	 *
	 * @param so
	 *            das betroffene Systemobjekt.
	 * @param datensatz
	 *            der aktuallisierten Analysedatensatz des Objekts
	 * @param datum
	 *            das aktuelle Datum.
	 */
	protected synchronized void fireNeuerMesswert(final SystemObjekt so,
			final OnlineDatensatz<? extends MesswertDatum> datensatz,
			final MesswertDatum datum) {
		final MesswertEvent e = new MesswertEvent(this, so, datensatz, datum);

		for (final MesswertListener l : listeners
				.getListeners(MesswertListener.class)) {
			l.messwertAktualisiert(e);
		}
	}

}
